@@ -5,19 +5,20 @@ from typing import Dict, Iterable
 
 from pymatgen.analysis.phase_diagram import PhaseDiagram
 from pymatgen.entries import Entry
+from tqdm import tqdm
 
 
-def expand_pd(
-    entries: Iterable[Entry],
-) -> Dict[str, PhaseDiagram]:
+def expand_pd(entries: Iterable[Entry], pbar: bool = False) -> Dict[str, PhaseDiagram]:
     """
     Helper method for generating a set of smaller phase diagrams for analyzing
-    thermodynamic staiblity in large chemical systems. This is necessary when
+    thermodynamic stability in large chemical systems. This is necessary when
     considering chemical systems which contain 10 or more elements, due to dimensional
     limitations of the Qhull algorithm.
 
     Args:
         entries ([Entry]): list of Entry objects for building phase diagram.
+        pbar (bool): whether to show a progress bar.
+
     Returns:
         Dictionary of PhaseDiagram objects indexed by chemical subsystem string;
         e.g. {"Li-Mn-O": <PhaseDiagram object>, "C-Y": <PhaseDiagram object>, ...}
@@ -29,7 +30,7 @@ def expand_pd(
         entries, key=lambda x: len(x.composition.elements), reverse=True
     )
 
-    for e in sorted_entries:
+    for e in tqdm(sorted_entries, disable=not pbar, desc="Building phase diagrams"):
         for chemsys in pd_dict:
             if set(e.composition.chemical_system.split("-")).issubset(
                 chemsys.split("-")
